# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
import dataclasses
import json  # type: ignore
import logging
from typing import Any, Callable, Dict, List, Optional, Sequence, Tuple, Union
import warnings

from google.api_core import exceptions as core_exceptions
from google.api_core import gapic_v1, rest_helpers, rest_streaming
from google.api_core import retry as retries
from google.auth import credentials as ga_credentials  # type: ignore
from google.auth.transport.requests import AuthorizedSession  # type: ignore
from google.longrunning import operations_pb2  # type: ignore
from google.protobuf import empty_pb2  # type: ignore
from google.protobuf import json_format
from requests import __version__ as requests_version

from google.cloud.chronicle_v1.types import entity

from .base import DEFAULT_CLIENT_INFO as BASE_DEFAULT_CLIENT_INFO
from .rest_base import _BaseEntityServiceRestTransport

try:
    OptionalRetry = Union[retries.Retry, gapic_v1.method._MethodDefault, None]
except AttributeError:  # pragma: NO COVER
    OptionalRetry = Union[retries.Retry, object, None]  # type: ignore

try:
    from google.api_core import client_logging  # type: ignore

    CLIENT_LOGGING_SUPPORTED = True  # pragma: NO COVER
except ImportError:  # pragma: NO COVER
    CLIENT_LOGGING_SUPPORTED = False

_LOGGER = logging.getLogger(__name__)

DEFAULT_CLIENT_INFO = gapic_v1.client_info.ClientInfo(
    gapic_version=BASE_DEFAULT_CLIENT_INFO.gapic_version,
    grpc_version=None,
    rest_version=f"requests@{requests_version}",
)


class EntityServiceRestInterceptor:
    """Interceptor for EntityService.

    Interceptors are used to manipulate requests, request metadata, and responses
    in arbitrary ways.
    Example use cases include:
    * Logging
    * Verifying requests according to service or custom semantics
    * Stripping extraneous information from responses

    These use cases and more can be enabled by injecting an
    instance of a custom subclass when constructing the EntityServiceRestTransport.

    .. code-block:: python
        class MyCustomEntityServiceInterceptor(EntityServiceRestInterceptor):
            def pre_create_watchlist(self, request, metadata):
                logging.log(f"Received request: {request}")
                return request, metadata

            def post_create_watchlist(self, response):
                logging.log(f"Received response: {response}")
                return response

            def pre_delete_watchlist(self, request, metadata):
                logging.log(f"Received request: {request}")
                return request, metadata

            def pre_get_watchlist(self, request, metadata):
                logging.log(f"Received request: {request}")
                return request, metadata

            def post_get_watchlist(self, response):
                logging.log(f"Received response: {response}")
                return response

            def pre_list_watchlists(self, request, metadata):
                logging.log(f"Received request: {request}")
                return request, metadata

            def post_list_watchlists(self, response):
                logging.log(f"Received response: {response}")
                return response

            def pre_update_watchlist(self, request, metadata):
                logging.log(f"Received request: {request}")
                return request, metadata

            def post_update_watchlist(self, response):
                logging.log(f"Received response: {response}")
                return response

        transport = EntityServiceRestTransport(interceptor=MyCustomEntityServiceInterceptor())
        client = EntityServiceClient(transport=transport)


    """

    def pre_create_watchlist(
        self,
        request: entity.CreateWatchlistRequest,
        metadata: Sequence[Tuple[str, Union[str, bytes]]],
    ) -> Tuple[entity.CreateWatchlistRequest, Sequence[Tuple[str, Union[str, bytes]]]]:
        """Pre-rpc interceptor for create_watchlist

        Override in a subclass to manipulate the request or metadata
        before they are sent to the EntityService server.
        """
        return request, metadata

    def post_create_watchlist(self, response: entity.Watchlist) -> entity.Watchlist:
        """Post-rpc interceptor for create_watchlist

        DEPRECATED. Please use the `post_create_watchlist_with_metadata`
        interceptor instead.

        Override in a subclass to read or manipulate the response
        after it is returned by the EntityService server but before
        it is returned to user code. This `post_create_watchlist` interceptor runs
        before the `post_create_watchlist_with_metadata` interceptor.
        """
        return response

    def post_create_watchlist_with_metadata(
        self,
        response: entity.Watchlist,
        metadata: Sequence[Tuple[str, Union[str, bytes]]],
    ) -> Tuple[entity.Watchlist, Sequence[Tuple[str, Union[str, bytes]]]]:
        """Post-rpc interceptor for create_watchlist

        Override in a subclass to read or manipulate the response or metadata after it
        is returned by the EntityService server but before it is returned to user code.

        We recommend only using this `post_create_watchlist_with_metadata`
        interceptor in new development instead of the `post_create_watchlist` interceptor.
        When both interceptors are used, this `post_create_watchlist_with_metadata` interceptor runs after the
        `post_create_watchlist` interceptor. The (possibly modified) response returned by
        `post_create_watchlist` will be passed to
        `post_create_watchlist_with_metadata`.
        """
        return response, metadata

    def pre_delete_watchlist(
        self,
        request: entity.DeleteWatchlistRequest,
        metadata: Sequence[Tuple[str, Union[str, bytes]]],
    ) -> Tuple[entity.DeleteWatchlistRequest, Sequence[Tuple[str, Union[str, bytes]]]]:
        """Pre-rpc interceptor for delete_watchlist

        Override in a subclass to manipulate the request or metadata
        before they are sent to the EntityService server.
        """
        return request, metadata

    def pre_get_watchlist(
        self,
        request: entity.GetWatchlistRequest,
        metadata: Sequence[Tuple[str, Union[str, bytes]]],
    ) -> Tuple[entity.GetWatchlistRequest, Sequence[Tuple[str, Union[str, bytes]]]]:
        """Pre-rpc interceptor for get_watchlist

        Override in a subclass to manipulate the request or metadata
        before they are sent to the EntityService server.
        """
        return request, metadata

    def post_get_watchlist(self, response: entity.Watchlist) -> entity.Watchlist:
        """Post-rpc interceptor for get_watchlist

        DEPRECATED. Please use the `post_get_watchlist_with_metadata`
        interceptor instead.

        Override in a subclass to read or manipulate the response
        after it is returned by the EntityService server but before
        it is returned to user code. This `post_get_watchlist` interceptor runs
        before the `post_get_watchlist_with_metadata` interceptor.
        """
        return response

    def post_get_watchlist_with_metadata(
        self,
        response: entity.Watchlist,
        metadata: Sequence[Tuple[str, Union[str, bytes]]],
    ) -> Tuple[entity.Watchlist, Sequence[Tuple[str, Union[str, bytes]]]]:
        """Post-rpc interceptor for get_watchlist

        Override in a subclass to read or manipulate the response or metadata after it
        is returned by the EntityService server but before it is returned to user code.

        We recommend only using this `post_get_watchlist_with_metadata`
        interceptor in new development instead of the `post_get_watchlist` interceptor.
        When both interceptors are used, this `post_get_watchlist_with_metadata` interceptor runs after the
        `post_get_watchlist` interceptor. The (possibly modified) response returned by
        `post_get_watchlist` will be passed to
        `post_get_watchlist_with_metadata`.
        """
        return response, metadata

    def pre_list_watchlists(
        self,
        request: entity.ListWatchlistsRequest,
        metadata: Sequence[Tuple[str, Union[str, bytes]]],
    ) -> Tuple[entity.ListWatchlistsRequest, Sequence[Tuple[str, Union[str, bytes]]]]:
        """Pre-rpc interceptor for list_watchlists

        Override in a subclass to manipulate the request or metadata
        before they are sent to the EntityService server.
        """
        return request, metadata

    def post_list_watchlists(
        self, response: entity.ListWatchlistsResponse
    ) -> entity.ListWatchlistsResponse:
        """Post-rpc interceptor for list_watchlists

        DEPRECATED. Please use the `post_list_watchlists_with_metadata`
        interceptor instead.

        Override in a subclass to read or manipulate the response
        after it is returned by the EntityService server but before
        it is returned to user code. This `post_list_watchlists` interceptor runs
        before the `post_list_watchlists_with_metadata` interceptor.
        """
        return response

    def post_list_watchlists_with_metadata(
        self,
        response: entity.ListWatchlistsResponse,
        metadata: Sequence[Tuple[str, Union[str, bytes]]],
    ) -> Tuple[entity.ListWatchlistsResponse, Sequence[Tuple[str, Union[str, bytes]]]]:
        """Post-rpc interceptor for list_watchlists

        Override in a subclass to read or manipulate the response or metadata after it
        is returned by the EntityService server but before it is returned to user code.

        We recommend only using this `post_list_watchlists_with_metadata`
        interceptor in new development instead of the `post_list_watchlists` interceptor.
        When both interceptors are used, this `post_list_watchlists_with_metadata` interceptor runs after the
        `post_list_watchlists` interceptor. The (possibly modified) response returned by
        `post_list_watchlists` will be passed to
        `post_list_watchlists_with_metadata`.
        """
        return response, metadata

    def pre_update_watchlist(
        self,
        request: entity.UpdateWatchlistRequest,
        metadata: Sequence[Tuple[str, Union[str, bytes]]],
    ) -> Tuple[entity.UpdateWatchlistRequest, Sequence[Tuple[str, Union[str, bytes]]]]:
        """Pre-rpc interceptor for update_watchlist

        Override in a subclass to manipulate the request or metadata
        before they are sent to the EntityService server.
        """
        return request, metadata

    def post_update_watchlist(self, response: entity.Watchlist) -> entity.Watchlist:
        """Post-rpc interceptor for update_watchlist

        DEPRECATED. Please use the `post_update_watchlist_with_metadata`
        interceptor instead.

        Override in a subclass to read or manipulate the response
        after it is returned by the EntityService server but before
        it is returned to user code. This `post_update_watchlist` interceptor runs
        before the `post_update_watchlist_with_metadata` interceptor.
        """
        return response

    def post_update_watchlist_with_metadata(
        self,
        response: entity.Watchlist,
        metadata: Sequence[Tuple[str, Union[str, bytes]]],
    ) -> Tuple[entity.Watchlist, Sequence[Tuple[str, Union[str, bytes]]]]:
        """Post-rpc interceptor for update_watchlist

        Override in a subclass to read or manipulate the response or metadata after it
        is returned by the EntityService server but before it is returned to user code.

        We recommend only using this `post_update_watchlist_with_metadata`
        interceptor in new development instead of the `post_update_watchlist` interceptor.
        When both interceptors are used, this `post_update_watchlist_with_metadata` interceptor runs after the
        `post_update_watchlist` interceptor. The (possibly modified) response returned by
        `post_update_watchlist` will be passed to
        `post_update_watchlist_with_metadata`.
        """
        return response, metadata

    def pre_cancel_operation(
        self,
        request: operations_pb2.CancelOperationRequest,
        metadata: Sequence[Tuple[str, Union[str, bytes]]],
    ) -> Tuple[
        operations_pb2.CancelOperationRequest, Sequence[Tuple[str, Union[str, bytes]]]
    ]:
        """Pre-rpc interceptor for cancel_operation

        Override in a subclass to manipulate the request or metadata
        before they are sent to the EntityService server.
        """
        return request, metadata

    def post_cancel_operation(self, response: None) -> None:
        """Post-rpc interceptor for cancel_operation

        Override in a subclass to manipulate the response
        after it is returned by the EntityService server but before
        it is returned to user code.
        """
        return response

    def pre_delete_operation(
        self,
        request: operations_pb2.DeleteOperationRequest,
        metadata: Sequence[Tuple[str, Union[str, bytes]]],
    ) -> Tuple[
        operations_pb2.DeleteOperationRequest, Sequence[Tuple[str, Union[str, bytes]]]
    ]:
        """Pre-rpc interceptor for delete_operation

        Override in a subclass to manipulate the request or metadata
        before they are sent to the EntityService server.
        """
        return request, metadata

    def post_delete_operation(self, response: None) -> None:
        """Post-rpc interceptor for delete_operation

        Override in a subclass to manipulate the response
        after it is returned by the EntityService server but before
        it is returned to user code.
        """
        return response

    def pre_get_operation(
        self,
        request: operations_pb2.GetOperationRequest,
        metadata: Sequence[Tuple[str, Union[str, bytes]]],
    ) -> Tuple[
        operations_pb2.GetOperationRequest, Sequence[Tuple[str, Union[str, bytes]]]
    ]:
        """Pre-rpc interceptor for get_operation

        Override in a subclass to manipulate the request or metadata
        before they are sent to the EntityService server.
        """
        return request, metadata

    def post_get_operation(
        self, response: operations_pb2.Operation
    ) -> operations_pb2.Operation:
        """Post-rpc interceptor for get_operation

        Override in a subclass to manipulate the response
        after it is returned by the EntityService server but before
        it is returned to user code.
        """
        return response

    def pre_list_operations(
        self,
        request: operations_pb2.ListOperationsRequest,
        metadata: Sequence[Tuple[str, Union[str, bytes]]],
    ) -> Tuple[
        operations_pb2.ListOperationsRequest, Sequence[Tuple[str, Union[str, bytes]]]
    ]:
        """Pre-rpc interceptor for list_operations

        Override in a subclass to manipulate the request or metadata
        before they are sent to the EntityService server.
        """
        return request, metadata

    def post_list_operations(
        self, response: operations_pb2.ListOperationsResponse
    ) -> operations_pb2.ListOperationsResponse:
        """Post-rpc interceptor for list_operations

        Override in a subclass to manipulate the response
        after it is returned by the EntityService server but before
        it is returned to user code.
        """
        return response


@dataclasses.dataclass
class EntityServiceRestStub:
    _session: AuthorizedSession
    _host: str
    _interceptor: EntityServiceRestInterceptor


class EntityServiceRestTransport(_BaseEntityServiceRestTransport):
    """REST backend synchronous transport for EntityService.

    EntityService contains apis for finding entities.

    This class defines the same methods as the primary client, so the
    primary client can load the underlying transport implementation
    and call it.

    It sends JSON representations of protocol buffers over HTTP/1.1
    """

    def __init__(
        self,
        *,
        host: str = "chronicle.googleapis.com",
        credentials: Optional[ga_credentials.Credentials] = None,
        credentials_file: Optional[str] = None,
        scopes: Optional[Sequence[str]] = None,
        client_cert_source_for_mtls: Optional[Callable[[], Tuple[bytes, bytes]]] = None,
        quota_project_id: Optional[str] = None,
        client_info: gapic_v1.client_info.ClientInfo = DEFAULT_CLIENT_INFO,
        always_use_jwt_access: Optional[bool] = False,
        url_scheme: str = "https",
        interceptor: Optional[EntityServiceRestInterceptor] = None,
        api_audience: Optional[str] = None,
    ) -> None:
        """Instantiate the transport.

        Args:
            host (Optional[str]):
                 The hostname to connect to (default: 'chronicle.googleapis.com').
            credentials (Optional[google.auth.credentials.Credentials]): The
                authorization credentials to attach to requests. These
                credentials identify the application to the service; if none
                are specified, the client will attempt to ascertain the
                credentials from the environment.

            credentials_file (Optional[str]): A file with credentials that can
                be loaded with :func:`google.auth.load_credentials_from_file`.
                This argument is ignored if ``channel`` is provided.
            scopes (Optional(Sequence[str])): A list of scopes. This argument is
                ignored if ``channel`` is provided.
            client_cert_source_for_mtls (Callable[[], Tuple[bytes, bytes]]): Client
                certificate to configure mutual TLS HTTP channel. It is ignored
                if ``channel`` is provided.
            quota_project_id (Optional[str]): An optional project to use for billing
                and quota.
            client_info (google.api_core.gapic_v1.client_info.ClientInfo):
                The client info used to send a user-agent string along with
                API requests. If ``None``, then default info will be used.
                Generally, you only need to set this if you are developing
                your own client library.
            always_use_jwt_access (Optional[bool]): Whether self signed JWT should
                be used for service account credentials.
            url_scheme: the protocol scheme for the API endpoint.  Normally
                "https", but for testing or local servers,
                "http" can be specified.
        """
        # Run the base constructor
        # TODO(yon-mg): resolve other ctor params i.e. scopes, quota, etc.
        # TODO: When custom host (api_endpoint) is set, `scopes` must *also* be set on the
        # credentials object
        super().__init__(
            host=host,
            credentials=credentials,
            client_info=client_info,
            always_use_jwt_access=always_use_jwt_access,
            url_scheme=url_scheme,
            api_audience=api_audience,
        )
        self._session = AuthorizedSession(
            self._credentials, default_host=self.DEFAULT_HOST
        )
        if client_cert_source_for_mtls:
            self._session.configure_mtls_channel(client_cert_source_for_mtls)
        self._interceptor = interceptor or EntityServiceRestInterceptor()
        self._prep_wrapped_messages(client_info)

    class _CreateWatchlist(
        _BaseEntityServiceRestTransport._BaseCreateWatchlist, EntityServiceRestStub
    ):
        def __hash__(self):
            return hash("EntityServiceRestTransport.CreateWatchlist")

        @staticmethod
        def _get_response(
            host,
            metadata,
            query_params,
            session,
            timeout,
            transcoded_request,
            body=None,
        ):
            uri = transcoded_request["uri"]
            method = transcoded_request["method"]
            headers = dict(metadata)
            headers["Content-Type"] = "application/json"
            response = getattr(session, method)(
                "{host}{uri}".format(host=host, uri=uri),
                timeout=timeout,
                headers=headers,
                params=rest_helpers.flatten_query_params(query_params, strict=True),
                data=body,
            )
            return response

        def __call__(
            self,
            request: entity.CreateWatchlistRequest,
            *,
            retry: OptionalRetry = gapic_v1.method.DEFAULT,
            timeout: Optional[float] = None,
            metadata: Sequence[Tuple[str, Union[str, bytes]]] = (),
        ) -> entity.Watchlist:
            r"""Call the create watchlist method over HTTP.

            Args:
                request (~.entity.CreateWatchlistRequest):
                    The request object. Request message for creating
                watchlist.
                retry (google.api_core.retry.Retry): Designation of what errors, if any,
                    should be retried.
                timeout (float): The timeout for this request.
                metadata (Sequence[Tuple[str, Union[str, bytes]]]): Key/value pairs which should be
                    sent along with the request as metadata. Normally, each value must be of type `str`,
                    but for metadata keys ending with the suffix `-bin`, the corresponding values must
                    be of type `bytes`.

            Returns:
                ~.entity.Watchlist:
                    A watchlist is a list of entities
                that allows for bulk operations over the
                included entities.

            """

            http_options = (
                _BaseEntityServiceRestTransport._BaseCreateWatchlist._get_http_options()
            )

            request, metadata = self._interceptor.pre_create_watchlist(
                request, metadata
            )
            transcoded_request = _BaseEntityServiceRestTransport._BaseCreateWatchlist._get_transcoded_request(
                http_options, request
            )

            body = _BaseEntityServiceRestTransport._BaseCreateWatchlist._get_request_body_json(
                transcoded_request
            )

            # Jsonify the query params
            query_params = _BaseEntityServiceRestTransport._BaseCreateWatchlist._get_query_params_json(
                transcoded_request
            )

            if CLIENT_LOGGING_SUPPORTED and _LOGGER.isEnabledFor(
                logging.DEBUG
            ):  # pragma: NO COVER
                request_url = "{host}{uri}".format(
                    host=self._host, uri=transcoded_request["uri"]
                )
                method = transcoded_request["method"]
                try:
                    request_payload = type(request).to_json(request)
                except:
                    request_payload = None
                http_request = {
                    "payload": request_payload,
                    "requestMethod": method,
                    "requestUrl": request_url,
                    "headers": dict(metadata),
                }
                _LOGGER.debug(
                    f"Sending request for google.cloud.chronicle_v1.EntityServiceClient.CreateWatchlist",
                    extra={
                        "serviceName": "google.cloud.chronicle.v1.EntityService",
                        "rpcName": "CreateWatchlist",
                        "httpRequest": http_request,
                        "metadata": http_request["headers"],
                    },
                )

            # Send the request
            response = EntityServiceRestTransport._CreateWatchlist._get_response(
                self._host,
                metadata,
                query_params,
                self._session,
                timeout,
                transcoded_request,
                body,
            )

            # In case of error, raise the appropriate core_exceptions.GoogleAPICallError exception
            # subclass.
            if response.status_code >= 400:
                raise core_exceptions.from_http_response(response)

            # Return the response
            resp = entity.Watchlist()
            pb_resp = entity.Watchlist.pb(resp)

            json_format.Parse(response.content, pb_resp, ignore_unknown_fields=True)

            resp = self._interceptor.post_create_watchlist(resp)
            response_metadata = [(k, str(v)) for k, v in response.headers.items()]
            resp, _ = self._interceptor.post_create_watchlist_with_metadata(
                resp, response_metadata
            )
            if CLIENT_LOGGING_SUPPORTED and _LOGGER.isEnabledFor(
                logging.DEBUG
            ):  # pragma: NO COVER
                try:
                    response_payload = entity.Watchlist.to_json(response)
                except:
                    response_payload = None
                http_response = {
                    "payload": response_payload,
                    "headers": dict(response.headers),
                    "status": response.status_code,
                }
                _LOGGER.debug(
                    "Received response for google.cloud.chronicle_v1.EntityServiceClient.create_watchlist",
                    extra={
                        "serviceName": "google.cloud.chronicle.v1.EntityService",
                        "rpcName": "CreateWatchlist",
                        "metadata": http_response["headers"],
                        "httpResponse": http_response,
                    },
                )
            return resp

    class _DeleteWatchlist(
        _BaseEntityServiceRestTransport._BaseDeleteWatchlist, EntityServiceRestStub
    ):
        def __hash__(self):
            return hash("EntityServiceRestTransport.DeleteWatchlist")

        @staticmethod
        def _get_response(
            host,
            metadata,
            query_params,
            session,
            timeout,
            transcoded_request,
            body=None,
        ):
            uri = transcoded_request["uri"]
            method = transcoded_request["method"]
            headers = dict(metadata)
            headers["Content-Type"] = "application/json"
            response = getattr(session, method)(
                "{host}{uri}".format(host=host, uri=uri),
                timeout=timeout,
                headers=headers,
                params=rest_helpers.flatten_query_params(query_params, strict=True),
            )
            return response

        def __call__(
            self,
            request: entity.DeleteWatchlistRequest,
            *,
            retry: OptionalRetry = gapic_v1.method.DEFAULT,
            timeout: Optional[float] = None,
            metadata: Sequence[Tuple[str, Union[str, bytes]]] = (),
        ):
            r"""Call the delete watchlist method over HTTP.

            Args:
                request (~.entity.DeleteWatchlistRequest):
                    The request object. Request message for deleting
                watchlist.
                retry (google.api_core.retry.Retry): Designation of what errors, if any,
                    should be retried.
                timeout (float): The timeout for this request.
                metadata (Sequence[Tuple[str, Union[str, bytes]]]): Key/value pairs which should be
                    sent along with the request as metadata. Normally, each value must be of type `str`,
                    but for metadata keys ending with the suffix `-bin`, the corresponding values must
                    be of type `bytes`.
            """

            http_options = (
                _BaseEntityServiceRestTransport._BaseDeleteWatchlist._get_http_options()
            )

            request, metadata = self._interceptor.pre_delete_watchlist(
                request, metadata
            )
            transcoded_request = _BaseEntityServiceRestTransport._BaseDeleteWatchlist._get_transcoded_request(
                http_options, request
            )

            # Jsonify the query params
            query_params = _BaseEntityServiceRestTransport._BaseDeleteWatchlist._get_query_params_json(
                transcoded_request
            )

            if CLIENT_LOGGING_SUPPORTED and _LOGGER.isEnabledFor(
                logging.DEBUG
            ):  # pragma: NO COVER
                request_url = "{host}{uri}".format(
                    host=self._host, uri=transcoded_request["uri"]
                )
                method = transcoded_request["method"]
                try:
                    request_payload = json_format.MessageToJson(request)
                except:
                    request_payload = None
                http_request = {
                    "payload": request_payload,
                    "requestMethod": method,
                    "requestUrl": request_url,
                    "headers": dict(metadata),
                }
                _LOGGER.debug(
                    f"Sending request for google.cloud.chronicle_v1.EntityServiceClient.DeleteWatchlist",
                    extra={
                        "serviceName": "google.cloud.chronicle.v1.EntityService",
                        "rpcName": "DeleteWatchlist",
                        "httpRequest": http_request,
                        "metadata": http_request["headers"],
                    },
                )

            # Send the request
            response = EntityServiceRestTransport._DeleteWatchlist._get_response(
                self._host,
                metadata,
                query_params,
                self._session,
                timeout,
                transcoded_request,
            )

            # In case of error, raise the appropriate core_exceptions.GoogleAPICallError exception
            # subclass.
            if response.status_code >= 400:
                raise core_exceptions.from_http_response(response)

    class _GetWatchlist(
        _BaseEntityServiceRestTransport._BaseGetWatchlist, EntityServiceRestStub
    ):
        def __hash__(self):
            return hash("EntityServiceRestTransport.GetWatchlist")

        @staticmethod
        def _get_response(
            host,
            metadata,
            query_params,
            session,
            timeout,
            transcoded_request,
            body=None,
        ):
            uri = transcoded_request["uri"]
            method = transcoded_request["method"]
            headers = dict(metadata)
            headers["Content-Type"] = "application/json"
            response = getattr(session, method)(
                "{host}{uri}".format(host=host, uri=uri),
                timeout=timeout,
                headers=headers,
                params=rest_helpers.flatten_query_params(query_params, strict=True),
            )
            return response

        def __call__(
            self,
            request: entity.GetWatchlistRequest,
            *,
            retry: OptionalRetry = gapic_v1.method.DEFAULT,
            timeout: Optional[float] = None,
            metadata: Sequence[Tuple[str, Union[str, bytes]]] = (),
        ) -> entity.Watchlist:
            r"""Call the get watchlist method over HTTP.

            Args:
                request (~.entity.GetWatchlistRequest):
                    The request object. Request message for getting a
                watchlist.
                retry (google.api_core.retry.Retry): Designation of what errors, if any,
                    should be retried.
                timeout (float): The timeout for this request.
                metadata (Sequence[Tuple[str, Union[str, bytes]]]): Key/value pairs which should be
                    sent along with the request as metadata. Normally, each value must be of type `str`,
                    but for metadata keys ending with the suffix `-bin`, the corresponding values must
                    be of type `bytes`.

            Returns:
                ~.entity.Watchlist:
                    A watchlist is a list of entities
                that allows for bulk operations over the
                included entities.

            """

            http_options = (
                _BaseEntityServiceRestTransport._BaseGetWatchlist._get_http_options()
            )

            request, metadata = self._interceptor.pre_get_watchlist(request, metadata)
            transcoded_request = _BaseEntityServiceRestTransport._BaseGetWatchlist._get_transcoded_request(
                http_options, request
            )

            # Jsonify the query params
            query_params = _BaseEntityServiceRestTransport._BaseGetWatchlist._get_query_params_json(
                transcoded_request
            )

            if CLIENT_LOGGING_SUPPORTED and _LOGGER.isEnabledFor(
                logging.DEBUG
            ):  # pragma: NO COVER
                request_url = "{host}{uri}".format(
                    host=self._host, uri=transcoded_request["uri"]
                )
                method = transcoded_request["method"]
                try:
                    request_payload = type(request).to_json(request)
                except:
                    request_payload = None
                http_request = {
                    "payload": request_payload,
                    "requestMethod": method,
                    "requestUrl": request_url,
                    "headers": dict(metadata),
                }
                _LOGGER.debug(
                    f"Sending request for google.cloud.chronicle_v1.EntityServiceClient.GetWatchlist",
                    extra={
                        "serviceName": "google.cloud.chronicle.v1.EntityService",
                        "rpcName": "GetWatchlist",
                        "httpRequest": http_request,
                        "metadata": http_request["headers"],
                    },
                )

            # Send the request
            response = EntityServiceRestTransport._GetWatchlist._get_response(
                self._host,
                metadata,
                query_params,
                self._session,
                timeout,
                transcoded_request,
            )

            # In case of error, raise the appropriate core_exceptions.GoogleAPICallError exception
            # subclass.
            if response.status_code >= 400:
                raise core_exceptions.from_http_response(response)

            # Return the response
            resp = entity.Watchlist()
            pb_resp = entity.Watchlist.pb(resp)

            json_format.Parse(response.content, pb_resp, ignore_unknown_fields=True)

            resp = self._interceptor.post_get_watchlist(resp)
            response_metadata = [(k, str(v)) for k, v in response.headers.items()]
            resp, _ = self._interceptor.post_get_watchlist_with_metadata(
                resp, response_metadata
            )
            if CLIENT_LOGGING_SUPPORTED and _LOGGER.isEnabledFor(
                logging.DEBUG
            ):  # pragma: NO COVER
                try:
                    response_payload = entity.Watchlist.to_json(response)
                except:
                    response_payload = None
                http_response = {
                    "payload": response_payload,
                    "headers": dict(response.headers),
                    "status": response.status_code,
                }
                _LOGGER.debug(
                    "Received response for google.cloud.chronicle_v1.EntityServiceClient.get_watchlist",
                    extra={
                        "serviceName": "google.cloud.chronicle.v1.EntityService",
                        "rpcName": "GetWatchlist",
                        "metadata": http_response["headers"],
                        "httpResponse": http_response,
                    },
                )
            return resp

    class _ListWatchlists(
        _BaseEntityServiceRestTransport._BaseListWatchlists, EntityServiceRestStub
    ):
        def __hash__(self):
            return hash("EntityServiceRestTransport.ListWatchlists")

        @staticmethod
        def _get_response(
            host,
            metadata,
            query_params,
            session,
            timeout,
            transcoded_request,
            body=None,
        ):
            uri = transcoded_request["uri"]
            method = transcoded_request["method"]
            headers = dict(metadata)
            headers["Content-Type"] = "application/json"
            response = getattr(session, method)(
                "{host}{uri}".format(host=host, uri=uri),
                timeout=timeout,
                headers=headers,
                params=rest_helpers.flatten_query_params(query_params, strict=True),
            )
            return response

        def __call__(
            self,
            request: entity.ListWatchlistsRequest,
            *,
            retry: OptionalRetry = gapic_v1.method.DEFAULT,
            timeout: Optional[float] = None,
            metadata: Sequence[Tuple[str, Union[str, bytes]]] = (),
        ) -> entity.ListWatchlistsResponse:
            r"""Call the list watchlists method over HTTP.

            Args:
                request (~.entity.ListWatchlistsRequest):
                    The request object. Request message for listing
                watchlists.
                retry (google.api_core.retry.Retry): Designation of what errors, if any,
                    should be retried.
                timeout (float): The timeout for this request.
                metadata (Sequence[Tuple[str, Union[str, bytes]]]): Key/value pairs which should be
                    sent along with the request as metadata. Normally, each value must be of type `str`,
                    but for metadata keys ending with the suffix `-bin`, the corresponding values must
                    be of type `bytes`.

            Returns:
                ~.entity.ListWatchlistsResponse:
                    Response message for listing
                watchlists.

            """

            http_options = (
                _BaseEntityServiceRestTransport._BaseListWatchlists._get_http_options()
            )

            request, metadata = self._interceptor.pre_list_watchlists(request, metadata)
            transcoded_request = _BaseEntityServiceRestTransport._BaseListWatchlists._get_transcoded_request(
                http_options, request
            )

            # Jsonify the query params
            query_params = _BaseEntityServiceRestTransport._BaseListWatchlists._get_query_params_json(
                transcoded_request
            )

            if CLIENT_LOGGING_SUPPORTED and _LOGGER.isEnabledFor(
                logging.DEBUG
            ):  # pragma: NO COVER
                request_url = "{host}{uri}".format(
                    host=self._host, uri=transcoded_request["uri"]
                )
                method = transcoded_request["method"]
                try:
                    request_payload = type(request).to_json(request)
                except:
                    request_payload = None
                http_request = {
                    "payload": request_payload,
                    "requestMethod": method,
                    "requestUrl": request_url,
                    "headers": dict(metadata),
                }
                _LOGGER.debug(
                    f"Sending request for google.cloud.chronicle_v1.EntityServiceClient.ListWatchlists",
                    extra={
                        "serviceName": "google.cloud.chronicle.v1.EntityService",
                        "rpcName": "ListWatchlists",
                        "httpRequest": http_request,
                        "metadata": http_request["headers"],
                    },
                )

            # Send the request
            response = EntityServiceRestTransport._ListWatchlists._get_response(
                self._host,
                metadata,
                query_params,
                self._session,
                timeout,
                transcoded_request,
            )

            # In case of error, raise the appropriate core_exceptions.GoogleAPICallError exception
            # subclass.
            if response.status_code >= 400:
                raise core_exceptions.from_http_response(response)

            # Return the response
            resp = entity.ListWatchlistsResponse()
            pb_resp = entity.ListWatchlistsResponse.pb(resp)

            json_format.Parse(response.content, pb_resp, ignore_unknown_fields=True)

            resp = self._interceptor.post_list_watchlists(resp)
            response_metadata = [(k, str(v)) for k, v in response.headers.items()]
            resp, _ = self._interceptor.post_list_watchlists_with_metadata(
                resp, response_metadata
            )
            if CLIENT_LOGGING_SUPPORTED and _LOGGER.isEnabledFor(
                logging.DEBUG
            ):  # pragma: NO COVER
                try:
                    response_payload = entity.ListWatchlistsResponse.to_json(response)
                except:
                    response_payload = None
                http_response = {
                    "payload": response_payload,
                    "headers": dict(response.headers),
                    "status": response.status_code,
                }
                _LOGGER.debug(
                    "Received response for google.cloud.chronicle_v1.EntityServiceClient.list_watchlists",
                    extra={
                        "serviceName": "google.cloud.chronicle.v1.EntityService",
                        "rpcName": "ListWatchlists",
                        "metadata": http_response["headers"],
                        "httpResponse": http_response,
                    },
                )
            return resp

    class _UpdateWatchlist(
        _BaseEntityServiceRestTransport._BaseUpdateWatchlist, EntityServiceRestStub
    ):
        def __hash__(self):
            return hash("EntityServiceRestTransport.UpdateWatchlist")

        @staticmethod
        def _get_response(
            host,
            metadata,
            query_params,
            session,
            timeout,
            transcoded_request,
            body=None,
        ):
            uri = transcoded_request["uri"]
            method = transcoded_request["method"]
            headers = dict(metadata)
            headers["Content-Type"] = "application/json"
            response = getattr(session, method)(
                "{host}{uri}".format(host=host, uri=uri),
                timeout=timeout,
                headers=headers,
                params=rest_helpers.flatten_query_params(query_params, strict=True),
                data=body,
            )
            return response

        def __call__(
            self,
            request: entity.UpdateWatchlistRequest,
            *,
            retry: OptionalRetry = gapic_v1.method.DEFAULT,
            timeout: Optional[float] = None,
            metadata: Sequence[Tuple[str, Union[str, bytes]]] = (),
        ) -> entity.Watchlist:
            r"""Call the update watchlist method over HTTP.

            Args:
                request (~.entity.UpdateWatchlistRequest):
                    The request object. Request message for updating
                watchlist.
                retry (google.api_core.retry.Retry): Designation of what errors, if any,
                    should be retried.
                timeout (float): The timeout for this request.
                metadata (Sequence[Tuple[str, Union[str, bytes]]]): Key/value pairs which should be
                    sent along with the request as metadata. Normally, each value must be of type `str`,
                    but for metadata keys ending with the suffix `-bin`, the corresponding values must
                    be of type `bytes`.

            Returns:
                ~.entity.Watchlist:
                    A watchlist is a list of entities
                that allows for bulk operations over the
                included entities.

            """

            http_options = (
                _BaseEntityServiceRestTransport._BaseUpdateWatchlist._get_http_options()
            )

            request, metadata = self._interceptor.pre_update_watchlist(
                request, metadata
            )
            transcoded_request = _BaseEntityServiceRestTransport._BaseUpdateWatchlist._get_transcoded_request(
                http_options, request
            )

            body = _BaseEntityServiceRestTransport._BaseUpdateWatchlist._get_request_body_json(
                transcoded_request
            )

            # Jsonify the query params
            query_params = _BaseEntityServiceRestTransport._BaseUpdateWatchlist._get_query_params_json(
                transcoded_request
            )

            if CLIENT_LOGGING_SUPPORTED and _LOGGER.isEnabledFor(
                logging.DEBUG
            ):  # pragma: NO COVER
                request_url = "{host}{uri}".format(
                    host=self._host, uri=transcoded_request["uri"]
                )
                method = transcoded_request["method"]
                try:
                    request_payload = type(request).to_json(request)
                except:
                    request_payload = None
                http_request = {
                    "payload": request_payload,
                    "requestMethod": method,
                    "requestUrl": request_url,
                    "headers": dict(metadata),
                }
                _LOGGER.debug(
                    f"Sending request for google.cloud.chronicle_v1.EntityServiceClient.UpdateWatchlist",
                    extra={
                        "serviceName": "google.cloud.chronicle.v1.EntityService",
                        "rpcName": "UpdateWatchlist",
                        "httpRequest": http_request,
                        "metadata": http_request["headers"],
                    },
                )

            # Send the request
            response = EntityServiceRestTransport._UpdateWatchlist._get_response(
                self._host,
                metadata,
                query_params,
                self._session,
                timeout,
                transcoded_request,
                body,
            )

            # In case of error, raise the appropriate core_exceptions.GoogleAPICallError exception
            # subclass.
            if response.status_code >= 400:
                raise core_exceptions.from_http_response(response)

            # Return the response
            resp = entity.Watchlist()
            pb_resp = entity.Watchlist.pb(resp)

            json_format.Parse(response.content, pb_resp, ignore_unknown_fields=True)

            resp = self._interceptor.post_update_watchlist(resp)
            response_metadata = [(k, str(v)) for k, v in response.headers.items()]
            resp, _ = self._interceptor.post_update_watchlist_with_metadata(
                resp, response_metadata
            )
            if CLIENT_LOGGING_SUPPORTED and _LOGGER.isEnabledFor(
                logging.DEBUG
            ):  # pragma: NO COVER
                try:
                    response_payload = entity.Watchlist.to_json(response)
                except:
                    response_payload = None
                http_response = {
                    "payload": response_payload,
                    "headers": dict(response.headers),
                    "status": response.status_code,
                }
                _LOGGER.debug(
                    "Received response for google.cloud.chronicle_v1.EntityServiceClient.update_watchlist",
                    extra={
                        "serviceName": "google.cloud.chronicle.v1.EntityService",
                        "rpcName": "UpdateWatchlist",
                        "metadata": http_response["headers"],
                        "httpResponse": http_response,
                    },
                )
            return resp

    @property
    def create_watchlist(
        self,
    ) -> Callable[[entity.CreateWatchlistRequest], entity.Watchlist]:
        # The return type is fine, but mypy isn't sophisticated enough to determine what's going on here.
        # In C++ this would require a dynamic_cast
        return self._CreateWatchlist(self._session, self._host, self._interceptor)  # type: ignore

    @property
    def delete_watchlist(
        self,
    ) -> Callable[[entity.DeleteWatchlistRequest], empty_pb2.Empty]:
        # The return type is fine, but mypy isn't sophisticated enough to determine what's going on here.
        # In C++ this would require a dynamic_cast
        return self._DeleteWatchlist(self._session, self._host, self._interceptor)  # type: ignore

    @property
    def get_watchlist(self) -> Callable[[entity.GetWatchlistRequest], entity.Watchlist]:
        # The return type is fine, but mypy isn't sophisticated enough to determine what's going on here.
        # In C++ this would require a dynamic_cast
        return self._GetWatchlist(self._session, self._host, self._interceptor)  # type: ignore

    @property
    def list_watchlists(
        self,
    ) -> Callable[[entity.ListWatchlistsRequest], entity.ListWatchlistsResponse]:
        # The return type is fine, but mypy isn't sophisticated enough to determine what's going on here.
        # In C++ this would require a dynamic_cast
        return self._ListWatchlists(self._session, self._host, self._interceptor)  # type: ignore

    @property
    def update_watchlist(
        self,
    ) -> Callable[[entity.UpdateWatchlistRequest], entity.Watchlist]:
        # The return type is fine, but mypy isn't sophisticated enough to determine what's going on here.
        # In C++ this would require a dynamic_cast
        return self._UpdateWatchlist(self._session, self._host, self._interceptor)  # type: ignore

    @property
    def cancel_operation(self):
        return self._CancelOperation(self._session, self._host, self._interceptor)  # type: ignore

    class _CancelOperation(
        _BaseEntityServiceRestTransport._BaseCancelOperation, EntityServiceRestStub
    ):
        def __hash__(self):
            return hash("EntityServiceRestTransport.CancelOperation")

        @staticmethod
        def _get_response(
            host,
            metadata,
            query_params,
            session,
            timeout,
            transcoded_request,
            body=None,
        ):
            uri = transcoded_request["uri"]
            method = transcoded_request["method"]
            headers = dict(metadata)
            headers["Content-Type"] = "application/json"
            response = getattr(session, method)(
                "{host}{uri}".format(host=host, uri=uri),
                timeout=timeout,
                headers=headers,
                params=rest_helpers.flatten_query_params(query_params, strict=True),
                data=body,
            )
            return response

        def __call__(
            self,
            request: operations_pb2.CancelOperationRequest,
            *,
            retry: OptionalRetry = gapic_v1.method.DEFAULT,
            timeout: Optional[float] = None,
            metadata: Sequence[Tuple[str, Union[str, bytes]]] = (),
        ) -> None:
            r"""Call the cancel operation method over HTTP.

            Args:
                request (operations_pb2.CancelOperationRequest):
                    The request object for CancelOperation method.
                retry (google.api_core.retry.Retry): Designation of what errors, if any,
                    should be retried.
                timeout (float): The timeout for this request.
                metadata (Sequence[Tuple[str, Union[str, bytes]]]): Key/value pairs which should be
                    sent along with the request as metadata. Normally, each value must be of type `str`,
                    but for metadata keys ending with the suffix `-bin`, the corresponding values must
                    be of type `bytes`.
            """

            http_options = (
                _BaseEntityServiceRestTransport._BaseCancelOperation._get_http_options()
            )

            request, metadata = self._interceptor.pre_cancel_operation(
                request, metadata
            )
            transcoded_request = _BaseEntityServiceRestTransport._BaseCancelOperation._get_transcoded_request(
                http_options, request
            )

            body = _BaseEntityServiceRestTransport._BaseCancelOperation._get_request_body_json(
                transcoded_request
            )

            # Jsonify the query params
            query_params = _BaseEntityServiceRestTransport._BaseCancelOperation._get_query_params_json(
                transcoded_request
            )

            if CLIENT_LOGGING_SUPPORTED and _LOGGER.isEnabledFor(
                logging.DEBUG
            ):  # pragma: NO COVER
                request_url = "{host}{uri}".format(
                    host=self._host, uri=transcoded_request["uri"]
                )
                method = transcoded_request["method"]
                try:
                    request_payload = json_format.MessageToJson(request)
                except:
                    request_payload = None
                http_request = {
                    "payload": request_payload,
                    "requestMethod": method,
                    "requestUrl": request_url,
                    "headers": dict(metadata),
                }
                _LOGGER.debug(
                    f"Sending request for google.cloud.chronicle_v1.EntityServiceClient.CancelOperation",
                    extra={
                        "serviceName": "google.cloud.chronicle.v1.EntityService",
                        "rpcName": "CancelOperation",
                        "httpRequest": http_request,
                        "metadata": http_request["headers"],
                    },
                )

            # Send the request
            response = EntityServiceRestTransport._CancelOperation._get_response(
                self._host,
                metadata,
                query_params,
                self._session,
                timeout,
                transcoded_request,
                body,
            )

            # In case of error, raise the appropriate core_exceptions.GoogleAPICallError exception
            # subclass.
            if response.status_code >= 400:
                raise core_exceptions.from_http_response(response)

            return self._interceptor.post_cancel_operation(None)

    @property
    def delete_operation(self):
        return self._DeleteOperation(self._session, self._host, self._interceptor)  # type: ignore

    class _DeleteOperation(
        _BaseEntityServiceRestTransport._BaseDeleteOperation, EntityServiceRestStub
    ):
        def __hash__(self):
            return hash("EntityServiceRestTransport.DeleteOperation")

        @staticmethod
        def _get_response(
            host,
            metadata,
            query_params,
            session,
            timeout,
            transcoded_request,
            body=None,
        ):
            uri = transcoded_request["uri"]
            method = transcoded_request["method"]
            headers = dict(metadata)
            headers["Content-Type"] = "application/json"
            response = getattr(session, method)(
                "{host}{uri}".format(host=host, uri=uri),
                timeout=timeout,
                headers=headers,
                params=rest_helpers.flatten_query_params(query_params, strict=True),
            )
            return response

        def __call__(
            self,
            request: operations_pb2.DeleteOperationRequest,
            *,
            retry: OptionalRetry = gapic_v1.method.DEFAULT,
            timeout: Optional[float] = None,
            metadata: Sequence[Tuple[str, Union[str, bytes]]] = (),
        ) -> None:
            r"""Call the delete operation method over HTTP.

            Args:
                request (operations_pb2.DeleteOperationRequest):
                    The request object for DeleteOperation method.
                retry (google.api_core.retry.Retry): Designation of what errors, if any,
                    should be retried.
                timeout (float): The timeout for this request.
                metadata (Sequence[Tuple[str, Union[str, bytes]]]): Key/value pairs which should be
                    sent along with the request as metadata. Normally, each value must be of type `str`,
                    but for metadata keys ending with the suffix `-bin`, the corresponding values must
                    be of type `bytes`.
            """

            http_options = (
                _BaseEntityServiceRestTransport._BaseDeleteOperation._get_http_options()
            )

            request, metadata = self._interceptor.pre_delete_operation(
                request, metadata
            )
            transcoded_request = _BaseEntityServiceRestTransport._BaseDeleteOperation._get_transcoded_request(
                http_options, request
            )

            # Jsonify the query params
            query_params = _BaseEntityServiceRestTransport._BaseDeleteOperation._get_query_params_json(
                transcoded_request
            )

            if CLIENT_LOGGING_SUPPORTED and _LOGGER.isEnabledFor(
                logging.DEBUG
            ):  # pragma: NO COVER
                request_url = "{host}{uri}".format(
                    host=self._host, uri=transcoded_request["uri"]
                )
                method = transcoded_request["method"]
                try:
                    request_payload = json_format.MessageToJson(request)
                except:
                    request_payload = None
                http_request = {
                    "payload": request_payload,
                    "requestMethod": method,
                    "requestUrl": request_url,
                    "headers": dict(metadata),
                }
                _LOGGER.debug(
                    f"Sending request for google.cloud.chronicle_v1.EntityServiceClient.DeleteOperation",
                    extra={
                        "serviceName": "google.cloud.chronicle.v1.EntityService",
                        "rpcName": "DeleteOperation",
                        "httpRequest": http_request,
                        "metadata": http_request["headers"],
                    },
                )

            # Send the request
            response = EntityServiceRestTransport._DeleteOperation._get_response(
                self._host,
                metadata,
                query_params,
                self._session,
                timeout,
                transcoded_request,
            )

            # In case of error, raise the appropriate core_exceptions.GoogleAPICallError exception
            # subclass.
            if response.status_code >= 400:
                raise core_exceptions.from_http_response(response)

            return self._interceptor.post_delete_operation(None)

    @property
    def get_operation(self):
        return self._GetOperation(self._session, self._host, self._interceptor)  # type: ignore

    class _GetOperation(
        _BaseEntityServiceRestTransport._BaseGetOperation, EntityServiceRestStub
    ):
        def __hash__(self):
            return hash("EntityServiceRestTransport.GetOperation")

        @staticmethod
        def _get_response(
            host,
            metadata,
            query_params,
            session,
            timeout,
            transcoded_request,
            body=None,
        ):
            uri = transcoded_request["uri"]
            method = transcoded_request["method"]
            headers = dict(metadata)
            headers["Content-Type"] = "application/json"
            response = getattr(session, method)(
                "{host}{uri}".format(host=host, uri=uri),
                timeout=timeout,
                headers=headers,
                params=rest_helpers.flatten_query_params(query_params, strict=True),
            )
            return response

        def __call__(
            self,
            request: operations_pb2.GetOperationRequest,
            *,
            retry: OptionalRetry = gapic_v1.method.DEFAULT,
            timeout: Optional[float] = None,
            metadata: Sequence[Tuple[str, Union[str, bytes]]] = (),
        ) -> operations_pb2.Operation:
            r"""Call the get operation method over HTTP.

            Args:
                request (operations_pb2.GetOperationRequest):
                    The request object for GetOperation method.
                retry (google.api_core.retry.Retry): Designation of what errors, if any,
                    should be retried.
                timeout (float): The timeout for this request.
                metadata (Sequence[Tuple[str, Union[str, bytes]]]): Key/value pairs which should be
                    sent along with the request as metadata. Normally, each value must be of type `str`,
                    but for metadata keys ending with the suffix `-bin`, the corresponding values must
                    be of type `bytes`.

            Returns:
                operations_pb2.Operation: Response from GetOperation method.
            """

            http_options = (
                _BaseEntityServiceRestTransport._BaseGetOperation._get_http_options()
            )

            request, metadata = self._interceptor.pre_get_operation(request, metadata)
            transcoded_request = _BaseEntityServiceRestTransport._BaseGetOperation._get_transcoded_request(
                http_options, request
            )

            # Jsonify the query params
            query_params = _BaseEntityServiceRestTransport._BaseGetOperation._get_query_params_json(
                transcoded_request
            )

            if CLIENT_LOGGING_SUPPORTED and _LOGGER.isEnabledFor(
                logging.DEBUG
            ):  # pragma: NO COVER
                request_url = "{host}{uri}".format(
                    host=self._host, uri=transcoded_request["uri"]
                )
                method = transcoded_request["method"]
                try:
                    request_payload = json_format.MessageToJson(request)
                except:
                    request_payload = None
                http_request = {
                    "payload": request_payload,
                    "requestMethod": method,
                    "requestUrl": request_url,
                    "headers": dict(metadata),
                }
                _LOGGER.debug(
                    f"Sending request for google.cloud.chronicle_v1.EntityServiceClient.GetOperation",
                    extra={
                        "serviceName": "google.cloud.chronicle.v1.EntityService",
                        "rpcName": "GetOperation",
                        "httpRequest": http_request,
                        "metadata": http_request["headers"],
                    },
                )

            # Send the request
            response = EntityServiceRestTransport._GetOperation._get_response(
                self._host,
                metadata,
                query_params,
                self._session,
                timeout,
                transcoded_request,
            )

            # In case of error, raise the appropriate core_exceptions.GoogleAPICallError exception
            # subclass.
            if response.status_code >= 400:
                raise core_exceptions.from_http_response(response)

            content = response.content.decode("utf-8")
            resp = operations_pb2.Operation()
            resp = json_format.Parse(content, resp)
            resp = self._interceptor.post_get_operation(resp)
            if CLIENT_LOGGING_SUPPORTED and _LOGGER.isEnabledFor(
                logging.DEBUG
            ):  # pragma: NO COVER
                try:
                    response_payload = json_format.MessageToJson(resp)
                except:
                    response_payload = None
                http_response = {
                    "payload": response_payload,
                    "headers": dict(response.headers),
                    "status": response.status_code,
                }
                _LOGGER.debug(
                    "Received response for google.cloud.chronicle_v1.EntityServiceAsyncClient.GetOperation",
                    extra={
                        "serviceName": "google.cloud.chronicle.v1.EntityService",
                        "rpcName": "GetOperation",
                        "httpResponse": http_response,
                        "metadata": http_response["headers"],
                    },
                )
            return resp

    @property
    def list_operations(self):
        return self._ListOperations(self._session, self._host, self._interceptor)  # type: ignore

    class _ListOperations(
        _BaseEntityServiceRestTransport._BaseListOperations, EntityServiceRestStub
    ):
        def __hash__(self):
            return hash("EntityServiceRestTransport.ListOperations")

        @staticmethod
        def _get_response(
            host,
            metadata,
            query_params,
            session,
            timeout,
            transcoded_request,
            body=None,
        ):
            uri = transcoded_request["uri"]
            method = transcoded_request["method"]
            headers = dict(metadata)
            headers["Content-Type"] = "application/json"
            response = getattr(session, method)(
                "{host}{uri}".format(host=host, uri=uri),
                timeout=timeout,
                headers=headers,
                params=rest_helpers.flatten_query_params(query_params, strict=True),
            )
            return response

        def __call__(
            self,
            request: operations_pb2.ListOperationsRequest,
            *,
            retry: OptionalRetry = gapic_v1.method.DEFAULT,
            timeout: Optional[float] = None,
            metadata: Sequence[Tuple[str, Union[str, bytes]]] = (),
        ) -> operations_pb2.ListOperationsResponse:
            r"""Call the list operations method over HTTP.

            Args:
                request (operations_pb2.ListOperationsRequest):
                    The request object for ListOperations method.
                retry (google.api_core.retry.Retry): Designation of what errors, if any,
                    should be retried.
                timeout (float): The timeout for this request.
                metadata (Sequence[Tuple[str, Union[str, bytes]]]): Key/value pairs which should be
                    sent along with the request as metadata. Normally, each value must be of type `str`,
                    but for metadata keys ending with the suffix `-bin`, the corresponding values must
                    be of type `bytes`.

            Returns:
                operations_pb2.ListOperationsResponse: Response from ListOperations method.
            """

            http_options = (
                _BaseEntityServiceRestTransport._BaseListOperations._get_http_options()
            )

            request, metadata = self._interceptor.pre_list_operations(request, metadata)
            transcoded_request = _BaseEntityServiceRestTransport._BaseListOperations._get_transcoded_request(
                http_options, request
            )

            # Jsonify the query params
            query_params = _BaseEntityServiceRestTransport._BaseListOperations._get_query_params_json(
                transcoded_request
            )

            if CLIENT_LOGGING_SUPPORTED and _LOGGER.isEnabledFor(
                logging.DEBUG
            ):  # pragma: NO COVER
                request_url = "{host}{uri}".format(
                    host=self._host, uri=transcoded_request["uri"]
                )
                method = transcoded_request["method"]
                try:
                    request_payload = json_format.MessageToJson(request)
                except:
                    request_payload = None
                http_request = {
                    "payload": request_payload,
                    "requestMethod": method,
                    "requestUrl": request_url,
                    "headers": dict(metadata),
                }
                _LOGGER.debug(
                    f"Sending request for google.cloud.chronicle_v1.EntityServiceClient.ListOperations",
                    extra={
                        "serviceName": "google.cloud.chronicle.v1.EntityService",
                        "rpcName": "ListOperations",
                        "httpRequest": http_request,
                        "metadata": http_request["headers"],
                    },
                )

            # Send the request
            response = EntityServiceRestTransport._ListOperations._get_response(
                self._host,
                metadata,
                query_params,
                self._session,
                timeout,
                transcoded_request,
            )

            # In case of error, raise the appropriate core_exceptions.GoogleAPICallError exception
            # subclass.
            if response.status_code >= 400:
                raise core_exceptions.from_http_response(response)

            content = response.content.decode("utf-8")
            resp = operations_pb2.ListOperationsResponse()
            resp = json_format.Parse(content, resp)
            resp = self._interceptor.post_list_operations(resp)
            if CLIENT_LOGGING_SUPPORTED and _LOGGER.isEnabledFor(
                logging.DEBUG
            ):  # pragma: NO COVER
                try:
                    response_payload = json_format.MessageToJson(resp)
                except:
                    response_payload = None
                http_response = {
                    "payload": response_payload,
                    "headers": dict(response.headers),
                    "status": response.status_code,
                }
                _LOGGER.debug(
                    "Received response for google.cloud.chronicle_v1.EntityServiceAsyncClient.ListOperations",
                    extra={
                        "serviceName": "google.cloud.chronicle.v1.EntityService",
                        "rpcName": "ListOperations",
                        "httpResponse": http_response,
                        "metadata": http_response["headers"],
                    },
                )
            return resp

    @property
    def kind(self) -> str:
        return "rest"

    def close(self):
        self._session.close()


__all__ = ("EntityServiceRestTransport",)
